# File: db/models.py
from dataclasses import dataclass
from typing import ClassVar

from matrx_orm import (
    BaseDTO,
    BaseManager,
    BigIntegerField,
    BooleanField,
    CharField,
    DateTimeField,
    DecimalField,
    FloatField,
    ForeignKey,
    IntegerField,
    JSONBField,
    Model,
    SmallIntegerField,
    TextField,
    UUIDField,
    model_registry,
)


class Users(Model):
    id = UUIDField(primary_key=True, null=False)
    email = CharField(null=False)

    _table_name = "users"
    _db_schema = "auth"
    _database = "supabase_automation_matrix"


class AiProvider(Model):
    id = UUIDField(primary_key=True, null=False)
    name = CharField()
    company_description = TextField()
    documentation_link = CharField()
    models_link = CharField()
    _inverse_foreign_keys: ClassVar[dict[str, dict[str, str]]] = {
        "ai_settings": {
            "from_model": "AiSettings",
            "from_field": "ai_provider",
            "referenced_field": "id",
            "related_name": "ai_settings",
        },
        "ai_model": {
            "from_model": "AiModel",
            "from_field": "model_provider",
            "referenced_field": "id",
            "related_name": "ai_model",
        },
    }
    _database = "supabase_automation_matrix"


class CxAgentMemory(Model):
    id = UUIDField(primary_key=True, null=False)
    user_id = ForeignKey(to_model=Users, to_column="id", to_schema="auth", null=False, unique=True)
    memory_type = TextField(null=False)
    scope = TextField(null=False, default="user", unique=True)
    scope_id = TextField(unique=True)
    key = TextField(null=False, unique=True)
    content = TextField(null=False)
    importance = FloatField(default=0.5)
    access_count = IntegerField(default=0)
    last_accessed_at = DateTimeField()
    expires_at = DateTimeField()
    metadata = JSONBField(null=False, default={})
    created_at = DateTimeField(null=False)
    updated_at = DateTimeField(null=False)
    deleted_at = DateTimeField()
    _inverse_foreign_keys: ClassVar[dict[str, dict[str, str]]] = {}
    _database = "supabase_automation_matrix"


class Prompts(Model):
    id = UUIDField(primary_key=True, null=False)
    created_at = DateTimeField(null=False)
    updated_at = DateTimeField()
    name = CharField()
    messages = JSONBField()
    variable_defaults = JSONBField()
    tools = JSONBField()
    user_id = ForeignKey(
        to_model=Users,
        to_column="id",
        to_schema="auth",
    )
    settings = JSONBField()
    description = TextField()
    _inverse_foreign_keys: ClassVar[dict[str, dict[str, str]]] = {
        "prompt_apps": {
            "from_model": "PromptApps",
            "from_field": "prompt_id",
            "referenced_field": "id",
            "related_name": "prompt_apps",
        },
        "system_prompts_new": {
            "from_model": "SystemPromptsNew",
            "from_field": "source_prompt_id",
            "referenced_field": "id",
            "related_name": "system_prompts_new",
        },
        "prompt_builtins": {
            "from_model": "PromptBuiltins",
            "from_field": "source_prompt_id",
            "referenced_field": "id",
            "related_name": "prompt_builtins",
        },
        "prompt_actions": {
            "from_model": "PromptActions",
            "from_field": "prompt_id",
            "referenced_field": "id",
            "related_name": "prompt_actions",
        },
        "system_prompts": {
            "from_model": "SystemPrompts",
            "from_field": "source_prompt_id",
            "referenced_field": "id",
            "related_name": "system_prompts",
        },
    }
    _database = "supabase_automation_matrix"


class ShortcutCategories(Model):
    id = UUIDField(primary_key=True, null=False)
    placement_type = TextField(null=False)
    parent_category_id = ForeignKey(
        to_model="ShortcutCategories",
        to_column="id",
    )
    label = TextField(null=False)
    description = TextField()
    icon_name = TextField(null=False, default="SquareMenu")
    color = TextField(default="zinc")
    sort_order = IntegerField(default=999)
    is_active = BooleanField(default=True)
    metadata = JSONBField(default={})
    enabled_contexts = JSONBField(default=["general"])
    _inverse_foreign_keys: ClassVar[dict[str, dict[str, str]]] = {
        "content_blocks": {
            "from_model": "ContentBlocks",
            "from_field": "category_id",
            "referenced_field": "id",
            "related_name": "content_blocks",
        },
        "prompt_shortcuts": {
            "from_model": "PromptShortcuts",
            "from_field": "category_id",
            "referenced_field": "id",
            "related_name": "prompt_shortcuts",
        },
        "system_prompts_new": {
            "from_model": "SystemPromptsNew",
            "from_field": "category_id",
            "referenced_field": "id",
            "related_name": "system_prompts_new",
        },
    }
    _database = "supabase_automation_matrix"


class Tools(Model):
    id = UUIDField(primary_key=True, null=False)
    name = TextField(null=False, unique=True)
    description = TextField(null=False)
    parameters = JSONBField(null=False)
    output_schema = JSONBField()
    annotations = JSONBField(default=[])
    function_path = TextField(null=False)
    category = TextField()
    tags = JSONBField()
    icon = CharField(max_length=255)
    is_active = BooleanField(default=True)
    version = TextField(default="1.0.0")
    created_at = DateTimeField()
    updated_at = DateTimeField()
    source_app = TextField(null=False, default="matrx_ai")
    _inverse_foreign_keys: ClassVar[dict[str, dict[str, str]]] = {
        "tool_test_samples": {
            "from_model": "ToolTestSamples",
            "from_field": "tool_id",
            "referenced_field": "id",
            "related_name": "tool_test_samples",
        },
        "tool_ui_components": {
            "from_model": "ToolUiComponents",
            "from_field": "tool_id",
            "referenced_field": "id",
            "related_name": "tool_ui_components",
        },
    }
    _database = "supabase_automation_matrix"


class UserTables(Model):
    id = UUIDField(primary_key=True, null=False)
    table_name = CharField(null=False, max_length=255, unique=True)
    description = TextField()
    version = IntegerField(null=False, default=1)
    user_id = ForeignKey(to_model=Users, to_column="id", to_schema="auth", null=False, unique=True)
    is_public = BooleanField(null=False, default=False)
    authenticated_read = BooleanField(null=False, default=False)
    created_at = DateTimeField(null=False)
    updated_at = DateTimeField(null=False)
    row_ordering_config = JSONBField()
    _inverse_foreign_keys: ClassVar[dict[str, dict[str, str]]] = {
        "table_data": {
            "from_model": "TableData",
            "from_field": "table_id",
            "referenced_field": "id",
            "related_name": "table_data",
        },
        "table_fields": {
            "from_model": "TableFields",
            "from_field": "table_id",
            "referenced_field": "id",
            "related_name": "table_fields",
        },
    }
    _database = "supabase_automation_matrix"


class AiModel(Model):
    id = UUIDField(primary_key=True, null=False)
    name = CharField(null=False)
    common_name = CharField()
    model_class = CharField(null=False)
    provider = CharField()
    endpoints = JSONBField()
    context_window = BigIntegerField()
    max_tokens = BigIntegerField()
    capabilities = JSONBField()
    controls = JSONBField()
    model_provider = ForeignKey(
        to_model=AiProvider,
        to_column="id",
    )
    is_deprecated = BooleanField(default=False)
    is_primary = BooleanField(default=False)
    is_premium = BooleanField(default=False)
    api_class = CharField()
    _inverse_foreign_keys: ClassVar[dict[str, dict[str, str]]] = {
        "ai_model_endpoint": {
            "from_model": "AiModelEndpoint",
            "from_field": "ai_model_id",
            "referenced_field": "id",
            "related_name": "ai_model_endpoint",
        },
        "recipe_model": {
            "from_model": "RecipeModel",
            "from_field": "ai_model",
            "referenced_field": "id",
            "related_name": "recipe_model",
        },
        "ai_settings": {
            "from_model": "AiSettings",
            "from_field": "ai_model",
            "referenced_field": "id",
            "related_name": "ai_settings",
        },
        "agent_conversations": {
            "from_model": "AgentConversations",
            "from_field": "model_id",
            "referenced_field": "id",
            "related_name": "agent_conversations",
        },
        "agent_requests": {
            "from_model": "AgentRequests",
            "from_field": "model_id",
            "referenced_field": "id",
            "related_name": "agent_requests",
        },
        "cx_conversation": {
            "from_model": "CxConversation",
            "from_field": "ai_model_id",
            "referenced_field": "id",
            "related_name": "cx_conversation",
        },
        "cx_user_request": {
            "from_model": "CxUserRequest",
            "from_field": "ai_model_id",
            "referenced_field": "id",
            "related_name": "cx_user_request",
        },
        "cx_request": {
            "from_model": "CxRequest",
            "from_field": "ai_model_id",
            "referenced_field": "id",
            "related_name": "cx_request",
        },
    }
    _database = "supabase_automation_matrix"


class ContentBlocks(Model):
    id = UUIDField(primary_key=True, null=False)
    block_id = CharField(null=False, max_length=100, unique=True)
    label = CharField(null=False, max_length=255)
    description = TextField()
    icon_name = CharField(null=False, max_length=100)
    template = TextField(null=False)
    sort_order = IntegerField(default=0)
    is_active = BooleanField(default=True)
    created_at = DateTimeField()
    updated_at = DateTimeField()
    category_id = ForeignKey(
        to_model=ShortcutCategories,
        to_column="id",
    )
    _inverse_foreign_keys: ClassVar[dict[str, dict[str, str]]] = {}
    _database = "supabase_automation_matrix"


class PromptBuiltins(Model):
    id = UUIDField(primary_key=True, null=False)
    created_at = DateTimeField(null=False)
    updated_at = DateTimeField(null=False)
    name = CharField(null=False)
    description = TextField()
    messages = JSONBField(null=False)
    variable_defaults = JSONBField()
    tools = JSONBField()
    settings = JSONBField()
    created_by_user_id = ForeignKey(
        to_model=Users,
        to_column="id",
        to_schema="auth",
    )
    is_active = BooleanField(null=False, default=True)
    source_prompt_id = ForeignKey(
        to_model=Prompts,
        to_column="id",
    )
    source_prompt_snapshot_at = DateTimeField()
    _inverse_foreign_keys: ClassVar[dict[str, dict[str, str]]] = {
        "prompt_shortcuts": {
            "from_model": "PromptShortcuts",
            "from_field": "prompt_builtin_id",
            "referenced_field": "id",
            "related_name": "prompt_shortcuts",
        },
        "prompt_actions": {
            "from_model": "PromptActions",
            "from_field": "prompt_builtin_id",
            "referenced_field": "id",
            "related_name": "prompt_actions",
        },
    }
    _database = "supabase_automation_matrix"


class TableData(Model):
    id = UUIDField(primary_key=True, null=False, unique=True)
    table_id = ForeignKey(to_model=UserTables, to_column="id", null=False, unique=True)
    data = JSONBField(null=False)
    user_id = ForeignKey(to_model=Users, to_column="id", to_schema="auth", null=False)
    is_public = BooleanField(null=False, default=False)
    authenticated_read = BooleanField(null=False, default=False)
    created_at = DateTimeField(null=False)
    updated_at = DateTimeField(null=False)
    _inverse_foreign_keys: ClassVar[dict[str, dict[str, str]]] = {}
    _database = "supabase_automation_matrix"


class CxConversation(Model):
    id = UUIDField(primary_key=True, null=False)
    user_id = ForeignKey(to_model=Users, to_column="id", to_schema="auth", null=False)
    title = TextField()
    system_instruction = TextField()
    config = JSONBField(null=False, default={})
    status = TextField(null=False, default="active")
    message_count = SmallIntegerField(null=False, default=0)
    forked_from_id = ForeignKey(
        to_model="CxConversation",
        to_column="id",
    )
    forked_at_position = SmallIntegerField()
    created_at = DateTimeField(null=False)
    updated_at = DateTimeField(null=False)
    deleted_at = DateTimeField()
    metadata = JSONBField(null=False, default={})
    ai_model_id = ForeignKey(
        to_model=AiModel,
        to_column="id",
    )
    parent_conversation_id = ForeignKey(
        to_model="CxConversation",
        to_column="id",
    )
    variables = JSONBField(null=False, default={})
    overrides = JSONBField(null=False, default={})
    _inverse_foreign_keys: ClassVar[dict[str, dict[str, str]]] = {
        "cx_tool_call": {
            "from_model": "CxToolCall",
            "from_field": "conversation_id",
            "referenced_field": "id",
            "related_name": "cx_tool_call",
        },
        "cx_message": {
            "from_model": "CxMessage",
            "from_field": "conversation_id",
            "referenced_field": "id",
            "related_name": "cx_message",
        },
        "cx_media": {
            "from_model": "CxMedia",
            "from_field": "conversation_id",
            "referenced_field": "id",
            "related_name": "cx_media",
        },
        "cx_user_request": {
            "from_model": "CxUserRequest",
            "from_field": "conversation_id",
            "referenced_field": "id",
            "related_name": "cx_user_request",
        },
        "cx_request": {
            "from_model": "CxRequest",
            "from_field": "conversation_id",
            "referenced_field": "id",
            "related_name": "cx_request",
        },
    }
    _database = "supabase_automation_matrix"


class CxMedia(Model):
    id = UUIDField(primary_key=True, null=False)
    conversation_id = ForeignKey(
        to_model=CxConversation,
        to_column="id",
    )
    user_id = ForeignKey(to_model=Users, to_column="id", to_schema="auth", null=False)
    kind = TextField(null=False)
    url = TextField(null=False)
    file_uri = TextField()
    mime_type = TextField()
    file_size_bytes = BigIntegerField()
    created_at = DateTimeField(null=False)
    deleted_at = DateTimeField()
    metadata = JSONBField(null=False, default={})
    _inverse_foreign_keys: ClassVar[dict[str, dict[str, str]]] = {}
    _database = "supabase_automation_matrix"


class CxMessage(Model):
    id = UUIDField(primary_key=True, null=False)
    conversation_id = ForeignKey(to_model=CxConversation, to_column="id", null=False)
    role = TextField(null=False)
    position = SmallIntegerField(null=False)
    status = TextField(null=False, default="active")
    content = JSONBField(null=False, default=[])
    created_at = DateTimeField(null=False)
    deleted_at = DateTimeField()
    metadata = JSONBField(null=False, default={})
    _inverse_foreign_keys: ClassVar[dict[str, dict[str, str]]] = {
        "cx_tool_call": {
            "from_model": "CxToolCall",
            "from_field": "message_id",
            "referenced_field": "id",
            "related_name": "cx_tool_call",
        }
    }
    _database = "supabase_automation_matrix"


class CxUserRequest(Model):
    id = UUIDField(primary_key=True, null=False)
    conversation_id = ForeignKey(to_model=CxConversation, to_column="id", null=False)
    user_id = ForeignKey(to_model=Users, to_column="id", to_schema="auth", null=False)
    trigger_message_position = SmallIntegerField()
    api_class = TextField()
    total_input_tokens = IntegerField(null=False, default=0)
    total_output_tokens = IntegerField(null=False, default=0)
    total_cached_tokens = IntegerField(null=False, default=0)
    total_tokens = IntegerField(null=False, default=0)
    total_cost = DecimalField(default=0)
    total_duration_ms = IntegerField()
    api_duration_ms = IntegerField()
    tool_duration_ms = IntegerField()
    iterations = SmallIntegerField(null=False, default=1)
    total_tool_calls = SmallIntegerField(null=False, default=0)
    status = TextField(null=False, default="pending")
    finish_reason = TextField()
    error = TextField()
    result_start_position = SmallIntegerField()
    result_end_position = SmallIntegerField()
    created_at = DateTimeField(null=False)
    completed_at = DateTimeField()
    deleted_at = DateTimeField()
    metadata = JSONBField(null=False, default={})
    ai_model_id = ForeignKey(
        to_model=AiModel,
        to_column="id",
    )
    _inverse_foreign_keys: ClassVar[dict[str, dict[str, str]]] = {
        "cx_tool_call": {
            "from_model": "CxToolCall",
            "from_field": "request_id",
            "referenced_field": "id",
            "related_name": "cx_tool_call",
        },
        "cx_request": {
            "from_model": "CxRequest",
            "from_field": "user_request_id",
            "referenced_field": "id",
            "related_name": "cx_request",
        },
    }
    _database = "supabase_automation_matrix"


class CxRequest(Model):
    id = UUIDField(primary_key=True, null=False)
    user_request_id = ForeignKey(to_model=CxUserRequest, to_column="id", null=False)
    conversation_id = ForeignKey(to_model=CxConversation, to_column="id", null=False)
    api_class = TextField()
    iteration = SmallIntegerField(null=False, default=1)
    input_tokens = IntegerField()
    output_tokens = IntegerField()
    cached_tokens = IntegerField()
    total_tokens = IntegerField()
    cost = DecimalField()
    api_duration_ms = IntegerField()
    tool_duration_ms = IntegerField()
    total_duration_ms = IntegerField()
    tool_calls_count = SmallIntegerField(default=0)
    tool_calls_details = JSONBField()
    finish_reason = TextField()
    response_id = TextField()
    created_at = DateTimeField(null=False)
    deleted_at = DateTimeField()
    metadata = JSONBField(null=False, default={})
    ai_model_id = ForeignKey(to_model=AiModel, to_column="id", null=False)
    _inverse_foreign_keys: ClassVar[dict[str, dict[str, str]]] = {}
    _database = "supabase_automation_matrix"


class CxToolCall(Model):
    id = UUIDField(primary_key=True, null=False)
    conversation_id = ForeignKey(to_model=CxConversation, to_column="id", null=False)
    message_id = ForeignKey(
        to_model=CxMessage,
        to_column="id",
    )
    user_id = ForeignKey(to_model=Users, to_column="id", to_schema="auth", null=False)
    request_id = ForeignKey(
        to_model=CxUserRequest,
        to_column="id",
    )
    tool_name = TextField(null=False)
    tool_type = TextField(null=False, default="local")
    call_id = TextField(null=False)
    status = TextField(null=False, default="pending")
    arguments = JSONBField(null=False, default={})
    success = BooleanField(null=False, default=True)
    output = TextField()
    output_type = TextField(default="text")
    is_error = BooleanField(default=False)
    error_type = TextField()
    error_message = TextField()
    duration_ms = IntegerField(null=False, default=0)
    started_at = DateTimeField(null=False)
    completed_at = DateTimeField(null=False)
    input_tokens = IntegerField(default=0)
    output_tokens = IntegerField(default=0)
    total_tokens = IntegerField(default=0)
    cost_usd = DecimalField(default=0)
    iteration = IntegerField(null=False, default=0)
    retry_count = IntegerField(default=0)
    parent_call_id = ForeignKey(
        to_model="CxToolCall",
        to_column="id",
    )
    execution_events = JSONBField(default=[])
    persist_key = TextField()
    file_path = TextField()
    metadata = JSONBField(null=False, default={})
    created_at = DateTimeField(null=False)
    deleted_at = DateTimeField()
    _inverse_foreign_keys: ClassVar[dict[str, dict[str, str]]] = {}
    _database = "supabase_automation_matrix"


model_registry.register_all(
    [
        AiProvider,
        CxAgentMemory,
        Prompts,
        ShortcutCategories,
        Tools,
        UserTables,
        AiModel,
        ContentBlocks,
        PromptBuiltins,
        TableData,
        CxConversation,
        CxMedia,
        CxMessage,
        CxUserRequest,
        CxRequest,
        CxToolCall,
        Users,
    ]
)


@dataclass
class AiProviderDTO(BaseDTO):
    id: str

    @classmethod
    async def from_model(cls, model: AiProvider):
        return cls(id=str(model.id))


class AiProviderManager(BaseManager):
    def __init__(self):
        super().__init__(AiProvider, AiProviderDTO)

    def _initialize_manager(self):
        super()._initialize_manager()

    async def _initialize_runtime_data(self, item: AiProvider) -> None:
        pass


@dataclass
class CxAgentMemoryDTO(BaseDTO):
    id: str

    @classmethod
    async def from_model(cls, model: CxAgentMemory):
        return cls(id=str(model.id))


class CxAgentMemoryManager(BaseManager):
    def __init__(self):
        super().__init__(CxAgentMemory, CxAgentMemoryDTO)

    def _initialize_manager(self):
        super()._initialize_manager()

    async def _initialize_runtime_data(self, item: CxAgentMemory) -> None:
        pass


@dataclass
class PromptsDTO(BaseDTO):
    id: str

    @classmethod
    async def from_model(cls, model: Prompts):
        return cls(id=str(model.id))


class PromptsManager(BaseManager):
    def __init__(self):
        super().__init__(Prompts, PromptsDTO)

    def _initialize_manager(self):
        super()._initialize_manager()

    async def _initialize_runtime_data(self, item: Prompts) -> None:
        pass


@dataclass
class ShortcutCategoriesDTO(BaseDTO):
    id: str

    @classmethod
    async def from_model(cls, model: ShortcutCategories):
        return cls(id=str(model.id))


class ShortcutCategoriesManager(BaseManager):
    def __init__(self):
        super().__init__(ShortcutCategories, ShortcutCategoriesDTO)

    def _initialize_manager(self):
        super()._initialize_manager()

    async def _initialize_runtime_data(self, item: ShortcutCategories) -> None:
        pass


@dataclass
class ToolsDTO(BaseDTO):
    id: str

    @classmethod
    async def from_model(cls, model: Tools):
        return cls(id=str(model.id))


class ToolsManager(BaseManager):
    def __init__(self):
        super().__init__(Tools, ToolsDTO)

    def _initialize_manager(self):
        super()._initialize_manager()

    async def _initialize_runtime_data(self, item: Tools) -> None:
        pass


@dataclass
class UserTablesDTO(BaseDTO):
    id: str

    @classmethod
    async def from_model(cls, model: UserTables):
        return cls(id=str(model.id))


class UserTablesManager(BaseManager):
    def __init__(self):
        super().__init__(UserTables, UserTablesDTO)

    def _initialize_manager(self):
        super()._initialize_manager()

    async def _initialize_runtime_data(self, item: UserTables) -> None:
        pass


@dataclass
class AiModelDTO(BaseDTO):
    id: str

    @classmethod
    async def from_model(cls, model: AiModel):
        return cls(id=str(model.id))


class AiModelManager(BaseManager):
    def __init__(self):
        super().__init__(AiModel, AiModelDTO)

    def _initialize_manager(self):
        super()._initialize_manager()

    async def _initialize_runtime_data(self, item: AiModel) -> None:
        pass


@dataclass
class ContentBlocksDTO(BaseDTO):
    id: str

    @classmethod
    async def from_model(cls, model: ContentBlocks):
        return cls(id=str(model.id))


class ContentBlocksManager(BaseManager):
    def __init__(self):
        super().__init__(ContentBlocks, ContentBlocksDTO)

    def _initialize_manager(self):
        super()._initialize_manager()

    async def _initialize_runtime_data(self, item: ContentBlocks) -> None:
        pass


@dataclass
class PromptBuiltinsDTO(BaseDTO):
    id: str

    @classmethod
    async def from_model(cls, model: PromptBuiltins):
        return cls(id=str(model.id))


class PromptBuiltinsManager(BaseManager):
    def __init__(self):
        super().__init__(PromptBuiltins, PromptBuiltinsDTO)

    def _initialize_manager(self):
        super()._initialize_manager()

    async def _initialize_runtime_data(self, item: PromptBuiltins) -> None:
        pass


@dataclass
class TableDataDTO(BaseDTO):
    id: str

    @classmethod
    async def from_model(cls, model: TableData):
        return cls(id=str(model.id))


class TableDataManager(BaseManager):
    def __init__(self):
        super().__init__(TableData, TableDataDTO)

    def _initialize_manager(self):
        super()._initialize_manager()

    async def _initialize_runtime_data(self, item: TableData) -> None:
        pass


@dataclass
class CxConversationDTO(BaseDTO):
    id: str

    @classmethod
    async def from_model(cls, model: CxConversation):
        return cls(id=str(model.id))


class CxConversationManager(BaseManager):
    def __init__(self):
        super().__init__(CxConversation, CxConversationDTO)

    def _initialize_manager(self):
        super()._initialize_manager()

    async def _initialize_runtime_data(self, item: CxConversation) -> None:
        pass


@dataclass
class CxMediaDTO(BaseDTO):
    id: str

    @classmethod
    async def from_model(cls, model: CxMedia):
        return cls(id=str(model.id))


class CxMediaManager(BaseManager):
    def __init__(self):
        super().__init__(CxMedia, CxMediaDTO)

    def _initialize_manager(self):
        super()._initialize_manager()

    async def _initialize_runtime_data(self, item: CxMedia) -> None:
        pass


@dataclass
class CxMessageDTO(BaseDTO):
    id: str

    @classmethod
    async def from_model(cls, model: CxMessage):
        return cls(id=str(model.id))


class CxMessageManager(BaseManager):
    def __init__(self):
        super().__init__(CxMessage, CxMessageDTO)

    def _initialize_manager(self):
        super()._initialize_manager()

    async def _initialize_runtime_data(self, item: CxMessage) -> None:
        pass


@dataclass
class CxUserRequestDTO(BaseDTO):
    id: str

    @classmethod
    async def from_model(cls, model: CxUserRequest):
        return cls(id=str(model.id))


class CxUserRequestManager(BaseManager):
    def __init__(self):
        super().__init__(CxUserRequest, CxUserRequestDTO)

    def _initialize_manager(self):
        super()._initialize_manager()

    async def _initialize_runtime_data(self, item: CxUserRequest) -> None:
        pass


@dataclass
class CxRequestDTO(BaseDTO):
    id: str

    @classmethod
    async def from_model(cls, model: CxRequest):
        return cls(id=str(model.id))


class CxRequestManager(BaseManager):
    def __init__(self):
        super().__init__(CxRequest, CxRequestDTO)

    def _initialize_manager(self):
        super()._initialize_manager()

    async def _initialize_runtime_data(self, item: CxRequest) -> None:
        pass


@dataclass
class CxToolCallDTO(BaseDTO):
    id: str

    @classmethod
    async def from_model(cls, model: CxToolCall):
        return cls(id=str(model.id))


class CxToolCallManager(BaseManager):
    def __init__(self):
        super().__init__(CxToolCall, CxToolCallDTO)

    def _initialize_manager(self):
        super()._initialize_manager()

    async def _initialize_runtime_data(self, item: CxToolCall) -> None:
        pass



from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from matrx_orm.client import SupabaseAuth, SupabaseClientConfig

# Module-level state — set once during initialize(), then read-only.
_client_mode: bool = False
_client_singleton: tuple[SupabaseClientConfig, SupabaseAuth] | None = None


def _setup(
    database_url: str | None = None,
    *,
    name: str = "supabase_automation_matrix",
    env_prefix: str = "SUPABASE_MATRIX",
    additional_schemas: list[str] | None = None,
    env_var_overrides: dict[str, str] | None = None,
    client_mode: bool = False,
    supabase_url: str = "",
    supabase_anon_key: str = "",
) -> None:
    """Connect the ORM to a PostgreSQL instance, or configure the Supabase client adapter.

    Called by matrx_ai.initialize() or directly when you want full control
    over the database configuration without relying on .env auto-loading.

    When client_mode=True, skips the asyncpg connection entirely and sets up
    the Supabase PostgREST adapter (anon key + user JWT + RLS). Use this when
    matrx-ai is embedded in a desktop app distributed to end users.
    """
    if client_mode:
        _setup_client_mode(supabase_url, supabase_anon_key)
        return

    from dotenv import load_dotenv
    from matrx_orm import register_database_from_env

    load_dotenv()
    register_database_from_env(
        name=name,
        env_prefix=env_prefix,
        additional_schemas=additional_schemas if additional_schemas is not None else ["auth"],
        env_var_overrides=env_var_overrides if env_var_overrides is not None else {"NAME": "SUPABASE_MATRIX_DATABASE_NAME"},
    )


def _setup_client_mode(url: str, anon_key: str) -> None:
    """Initialize the Supabase PostgREST adapter for client-side (desktop) use.

    Stores a (SupabaseClientConfig, SupabaseAuth) singleton that can be
    retrieved via get_client_singleton(). No asyncpg connection is opened.
    The anon key is safe to embed in desktop apps — per-user RLS policies
    enforce data isolation via the user's JWT.
    """
    from matrx_orm.client import SupabaseAuth, SupabaseClientConfig

    global _client_mode, _client_singleton

    if not url:
        raise ValueError(
            "client_mode=True requires supabase_url. "
            "Set SUPABASE_URL in .env or pass it explicitly to initialize()."
        )
    if not anon_key:
        raise ValueError(
            "client_mode=True requires supabase_anon_key. "
            "Set SUPABASE_PUBLISHABLE_KEY in .env or pass it explicitly to initialize()."
        )

    config = SupabaseClientConfig(url=url, anon_key=anon_key)
    auth = SupabaseAuth(config)
    _client_singleton = (config, auth)
    _client_mode = True


def is_client_mode() -> bool:
    """Return True if matrx-ai was initialized in client (PostgREST) mode."""
    return _client_mode


def get_client_singleton() -> tuple[SupabaseClientConfig, SupabaseAuth]:
    """Return the (SupabaseClientConfig, SupabaseAuth) singleton.

    Raises RuntimeError if not initialized in client mode.
    """
    if _client_singleton is None:
        raise RuntimeError(
            "matrx-ai is not initialized in client mode. "
            "Call matrx_ai.initialize(client_mode=True, ...) first."
        )
    return _client_singleton



# # File: db/managers/ai_model.py
# from __future__ import annotations

# from dataclasses import dataclass
# from typing import Any

# from matrx_orm import BaseDTO, BaseManager, ModelView

# from matrx_ai.db.models import AiModel

# # ---------------------------------------------------------------------------
# # ModelView (new) — opt-in projection layer.
# # Stores results flat on the model instance; no duplication, no nesting.
# # To activate: set view_class = AiModelView on your manager subclass,
# # or pass view_class=AiModelView to super().__init__().
# # When active, the DTO path below is skipped automatically.
# # ---------------------------------------------------------------------------

# class AiModelView(ModelView[AiModel]):
#     """
#     Declarative view for AiModel.

#     Configure what gets fetched and shaped automatically on every load:

#       prefetch    — relation names to fetch concurrently (FK / IFK / M2M)
#       exclude     — model field names to omit from to_dict() output
#       inline_fk   — replace FK id columns with the full related object
#                     e.g. {"customer_id": "customer"}

#     Add async methods (no leading underscore) for computed fields:

#         async def display_name(self, model: AiModel) -> str:
#             return model.name.title()
#     """

#     prefetch: list[str] = []
#     exclude: list[str] = []
#     inline_fk: dict[str, str] = {}

#     # ------------------------------------------------------------------ #
#     # Computed fields — add async methods below.                          #
#     # Each method receives the model instance and returns a plain value.  #
#     # Errors in computed fields are logged and stored as None —           #
#     # they never abort the load.                                          #
#     # ------------------------------------------------------------------ #


# # ---------------------------------------------------------------------------
# # BaseDTO (default) — active by default, fully backward compatible.
# # Extend _process_core_data / _process_metadata with your business logic.
# # When you are ready to migrate to the View above, set view_class on your
# # manager subclass and this DTO will be bypassed automatically.
# # ---------------------------------------------------------------------------

# @dataclass
# class AiModelDTO(BaseDTO[AiModel]):
#     id: str

#     async def _initialize_dto(self, model: AiModel) -> None:
#         '''Override to populate DTO fields from the model.'''
#         self.id = str(model.id)
#         await self._process_core_data(model)
#         await self._process_metadata(model)
#         await self._initial_validation(model)
#         self.initialized = True

#     async def _process_core_data(self, model: AiModel) -> None:
#         '''Process core data from the model item.'''
#         pass

#     async def _process_metadata(self, model: AiModel) -> None:
#         '''Process metadata from the model item.'''
#         pass

#     async def _initial_validation(self, model: AiModel) -> None:
#         '''Validate fields from the model item.'''
#         pass

#     async def _final_validation(self) -> bool:
#         '''Final validation of the model item.'''
#         return True

#     async def get_validated_dict(self) -> dict[str, Any]:
#         '''Get the validated dictionary.'''
#         await self._final_validation()
#         return self.to_dict()


# # ---------------------------------------------------------------------------
# # Manager — DTO is active by default for full backward compatibility.
# # To switch to the View (opt-in):
# #   1. Quick: set view_class = AiModelView  (replaces DTO automatically)
# #   2. Explicit: super().__init__(AiModel, view_class=AiModelView)
# # ---------------------------------------------------------------------------

# class AiModelBase(BaseManager[AiModel]):
#     view_class = None  # DTO is used by default; set to AiModelView to opt in

#     def __init__(
#         self,
#         dto_class: type[Any] | None = None,
#         view_class: type[Any] | None = None,
#     ) -> None:
#         if view_class is not None:
#             self.view_class = view_class
#         super().__init__(AiModel, dto_class=dto_class or AiModelDTO)

#     def _initialize_manager(self) -> None:
#         super()._initialize_manager()

#     async def _initialize_runtime_data(self, item: AiModel) -> None:
#         pass

#     async def create_ai_model(self, **data: Any) -> AiModel:
#         return await self.create_item(**data)

#     async def delete_ai_model(self, id: Any) -> bool:
#         return await self.delete_item(id)

#     async def get_ai_model_with_all_related(self, id: Any) -> tuple[AiModel, Any]:
#         return await self.get_item_with_all_related(id)

#     async def load_ai_model_by_id(self, id: Any) -> AiModel:
#         return await self.load_by_id(id)

#     async def load_ai_model(self, use_cache: bool = True, **kwargs: Any) -> AiModel:
#         return await self.load_item(use_cache, **kwargs)

#     async def update_ai_model(self, id: Any, **updates: Any) -> AiModel:
#         return await self.update_item(id, **updates)

#     async def load_ai_models(self, **kwargs: Any) -> list[AiModel]:
#         return await self.load_items(**kwargs)

#     async def filter_ai_models(self, **kwargs: Any) -> list[AiModel]:
#         return await self.filter_items(**kwargs)

#     async def get_or_create_ai_model(self, defaults: dict[str, Any] | None = None, **kwargs: Any) -> AiModel | None:
#         return await self.get_or_create(defaults, **kwargs)

#     async def get_ai_model_with_ai_provider(self, id: Any) -> tuple[Any, Any]:
#         return await self.get_item_with_related(id, 'ai_provider')

#     async def get_ai_models_with_ai_provider(self) -> list[Any]:
#         return await self.get_items_with_related('ai_provider')

#     async def get_ai_model_with_ai_model_endpoint(self, id: Any) -> tuple[Any, Any]:
#         return await self.get_item_with_related(id, 'ai_model_endpoint')

#     async def get_ai_models_with_ai_model_endpoint(self) -> list[Any]:
#         return await self.get_items_with_related('ai_model_endpoint')

#     async def get_ai_model_with_ai_settings(self, id: Any) -> tuple[Any, Any]:
#         return await self.get_item_with_related(id, 'ai_settings')

#     async def get_ai_models_with_ai_settings(self) -> list[Any]:
#         return await self.get_items_with_related('ai_settings')

#     async def get_ai_model_with_recipe_model(self, id: Any) -> tuple[Any, Any]:
#         return await self.get_item_with_related(id, 'recipe_model')

#     async def get_ai_models_with_recipe_model(self) -> list[Any]:
#         return await self.get_items_with_related('recipe_model')

#     async def load_ai_models_by_name(self, name: Any) -> list[Any]:
#         return await self.load_items(name=name)

#     async def filter_ai_models_by_name(self, name: Any) -> list[Any]:
#         return await self.filter_items(name=name)

#     async def load_ai_models_by_common_name(self, common_name: Any) -> list[Any]:
#         return await self.load_items(common_name=common_name)

#     async def filter_ai_models_by_common_name(self, common_name: Any) -> list[Any]:
#         return await self.filter_items(common_name=common_name)

#     async def load_ai_models_by_provider(self, provider: Any) -> list[Any]:
#         return await self.load_items(provider=provider)

#     async def filter_ai_models_by_provider(self, provider: Any) -> list[Any]:
#         return await self.filter_items(provider=provider)

#     async def load_ai_models_by_model_class(self, model_class: Any) -> list[Any]:
#         return await self.load_items(model_class=model_class)

#     async def filter_ai_models_by_model_class(self, model_class: Any) -> list[Any]:
#         return await self.filter_items(model_class=model_class)

#     async def load_ai_models_by_model_provider(self, model_provider: Any) -> list[Any]:
#         return await self.load_items(model_provider=model_provider)

#     async def filter_ai_models_by_model_provider(self, model_provider: Any) -> list[Any]:
#         return await self.filter_items(model_provider=model_provider)

#     async def load_ai_models_by_ids(self, ids: list[Any]) -> list[Any]:
#         return await self.load_items_by_ids(ids)

#     def add_computed_field(self, field: str) -> None:
#         super().add_computed_field(field)

#     def add_relation_field(self, field: str) -> None:
#         super().add_relation_field(field)

#     @property
#     def active_ai_model_ids(self) -> set[Any]:
#         return self.active_item_ids



# class AiModelManager(AiModelBase):
#     _instance: AiModelManager | None = None

#     def __new__(cls, *args: Any, **kwargs: Any) -> AiModelManager:
#         if cls._instance is None:
#             cls._instance = super().__new__(cls)
#         return cls._instance

#     def __init__(self) -> None:
#         super().__init__()

#     async def _initialize_runtime_data(self, item: AiModel) -> None:
#         pass

# _ai_model_manager_instance: AiModelManager | None = None


# def get_ai_model_manager() -> AiModelManager:
#     global _ai_model_manager_instance
#     if _ai_model_manager_instance is None:
#         _ai_model_manager_instance = AiModelManager()
#     return _ai_model_manager_instance


"""
matrx-ai — Matrx AI Engine

Core FastAPI backend for AI orchestration across 7 providers, multi-turn
conversations, tool execution, and streaming via SSE/NDJSON.

Quickstart (library mode — server, direct asyncpg):
    import matrx_ai
    matrx_ai.initialize()          # connects DB from env vars
    from matrx_ai.orchestrator.executor import execute_ai_request

Quickstart (client mode — desktop app, Supabase PostgREST + RLS):
    import matrx_ai
    matrx_ai.initialize(
        client_mode=True,
        supabase_url="https://abc123.supabase.co",
        supabase_anon_key="eyJ...",
    )
    # No asyncpg connection; all DB access goes through the user's JWT + RLS.

Quickstart (server mode — requires pip install matrx-ai[server]):
    matrx-ai-server                # CLI entry point
    # or programmatically:
    from matrx_ai.app.main import start
    start()
"""

from __future__ import annotations

__version__ = "0.1.19"


def _get_version() -> str:
    try:
        from importlib.metadata import version

        return version("matrx-ai")
    except Exception:
        return __version__


__version__ = _get_version()

_initialized = False


def initialize(
    database_url: str | None = None,
    *,
    db_name: str = "supabase_automation_matrix",
    db_env_prefix: str = "SUPABASE_MATRIX",
    db_additional_schemas: list[str] | None = None,
    db_env_var_overrides: dict[str, str] | None = None,
    client_mode: bool = False,
    supabase_url: str = "",
    supabase_anon_key: str = "",
) -> None:
    """Initialize the matrx-ai library.

    Server mode (default): Connects the ORM directly to PostgreSQL via asyncpg.
    Reads connection parameters from environment variables via env_prefix.

    Client mode: Skips asyncpg entirely. Sets up the Supabase PostgREST adapter
    (anon key + user JWT + RLS policies). Use when matrx-ai is embedded in a
    desktop application distributed to end users — no DB credentials are shipped.

    Args:
        database_url: Optional explicit DSN (server mode only). When None, reads
            connection parameters from environment variables via env_prefix.
        db_name: ORM connection alias (default: "supabase_automation_matrix").
        db_env_prefix: Env var prefix for DB config (default: "SUPABASE_MATRIX").
        db_additional_schemas: Extra PostgreSQL schemas to expose (default: ["auth"]).
        db_env_var_overrides: Env var name overrides passed to register_database_from_env.
        client_mode: When True, uses Supabase PostgREST instead of asyncpg.
            Requires supabase_url and supabase_anon_key.
        supabase_url: Supabase project URL (client mode only).
            e.g. "https://abc123.supabase.co"
        supabase_anon_key: Supabase publishable anon key (client mode only).
            Safe to embed in desktop apps — RLS enforces per-user data isolation.
    """
    global _initialized
    if _initialized:
        return

    from matrx_ai.db import _setup

    _setup(
        database_url=database_url,
        name=db_name,
        env_prefix=db_env_prefix,
        additional_schemas=db_additional_schemas,
        env_var_overrides=db_env_var_overrides,
        client_mode=client_mode,
        supabase_url=supabase_url,
        supabase_anon_key=supabase_anon_key,
    )
    _initialized = True


def is_client_mode() -> bool:
    """Return True if matrx-ai was initialized in client (PostgREST/RLS) mode.

    Use this to guard any code that would otherwise issue asyncpg queries.
    """
    from matrx_ai.db import is_client_mode as _is_client_mode
    return _is_client_mode()



# File: db/managers/content_blocks.py
from __future__ import annotations

from dataclasses import dataclass
from typing import Any

from matrx_orm import BaseDTO, BaseManager, ModelView

from matrx_ai.db.models import ContentBlocks

# ---------------------------------------------------------------------------
# ModelView (new) — opt-in projection layer.
# Stores results flat on the model instance; no duplication, no nesting.
# To activate: set view_class = ContentBlocksView on your manager subclass,
# or pass view_class=ContentBlocksView to super().__init__().
# When active, the DTO path below is skipped automatically.
# ---------------------------------------------------------------------------

class ContentBlocksView(ModelView[ContentBlocks]):
    """
    Declarative view for ContentBlocks.

    Configure what gets fetched and shaped automatically on every load:

      prefetch    — relation names to fetch concurrently (FK / IFK / M2M)
      exclude     — model field names to omit from to_dict() output
      inline_fk   — replace FK id columns with the full related object
                    e.g. {"customer_id": "customer"}

    Add async methods (no leading underscore) for computed fields:

        async def display_name(self, model: ContentBlocks) -> str:
            return model.name.title()
    """

    prefetch: list[str] = []
    exclude: list[str] = []
    inline_fk: dict[str, str] = {}

    # ------------------------------------------------------------------ #
    # Computed fields — add async methods below.                          #
    # Each method receives the model instance and returns a plain value.  #
    # Errors in computed fields are logged and stored as None —           #
    # they never abort the load.                                          #
    # ------------------------------------------------------------------ #


# ---------------------------------------------------------------------------
# BaseDTO (default) — active by default, fully backward compatible.
# Extend _process_core_data / _process_metadata with your business logic.
# When you are ready to migrate to the View above, set view_class on your
# manager subclass and this DTO will be bypassed automatically.
# ---------------------------------------------------------------------------

@dataclass
class ContentBlocksDTO(BaseDTO[ContentBlocks]):
    id: str

    async def _initialize_dto(self, model: ContentBlocks) -> None:
        '''Override to populate DTO fields from the model.'''
        self.id = str(model.id)
        await self._process_core_data(model)
        await self._process_metadata(model)
        await self._initial_validation(model)
        self.initialized = True

    async def _process_core_data(self, model: ContentBlocks) -> None:
        '''Process core data from the model item.'''
        pass

    async def _process_metadata(self, model: ContentBlocks) -> None:
        '''Process metadata from the model item.'''
        pass

    async def _initial_validation(self, model: ContentBlocks) -> None:
        '''Validate fields from the model item.'''
        pass

    async def _final_validation(self) -> bool:
        '''Final validation of the model item.'''
        return True

    async def get_validated_dict(self) -> dict[str, Any]:
        '''Get the validated dictionary.'''
        await self._final_validation()
        return self.to_dict()


# ---------------------------------------------------------------------------
# Manager — DTO is active by default for full backward compatibility.
# To switch to the View (opt-in):
#   1. Quick: set view_class = ContentBlocksView  (replaces DTO automatically)
#   2. Explicit: super().__init__(ContentBlocks, view_class=ContentBlocksView)
# ---------------------------------------------------------------------------

class ContentBlocksBase(BaseManager[ContentBlocks]):
    view_class = None  # DTO is used by default; set to ContentBlocksView to opt in

    def __init__(
        self,
        dto_class: type[Any] | None = None,
        view_class: type[Any] | None = None,
    ) -> None:
        if view_class is not None:
            self.view_class = view_class
        super().__init__(ContentBlocks, dto_class=dto_class or ContentBlocksDTO)

    def _initialize_manager(self) -> None:
        super()._initialize_manager()

    async def _initialize_runtime_data(self, item: ContentBlocks) -> None:
        pass

    async def create_content_blocks(self, **data: Any) -> ContentBlocks:
        return await self.create_item(**data)

    async def delete_content_blocks(self, id: Any) -> bool:
        return await self.delete_item(id)

    async def get_content_blocks_with_all_related(self, id: Any) -> tuple[ContentBlocks, Any]:
        return await self.get_item_with_all_related(id)

    async def load_content_blocks_by_id(self, id: Any) -> ContentBlocks:
        return await self.load_by_id(id)

    async def load_content_blocks(self, use_cache: bool = True, **kwargs: Any) -> ContentBlocks:
        return await self.load_item(use_cache, **kwargs)

    async def update_content_blocks(self, id: Any, **updates: Any) -> ContentBlocks:
        return await self.update_item(id, **updates)

    async def load_content_block(self, **kwargs: Any) -> list[ContentBlocks]:
        return await self.load_items(**kwargs)

    async def filter_content_block(self, **kwargs: Any) -> list[ContentBlocks]:
        return await self.filter_items(**kwargs)

    async def get_or_create_content_blocks(self, defaults: dict[str, Any] | None = None, **kwargs: Any) -> ContentBlocks | None:
        return await self.get_or_create(defaults, **kwargs)

    async def get_content_blocks_with_shortcut_categories(self, id: Any) -> tuple[Any, Any]:
        return await self.get_item_with_related(id, 'shortcut_categories')

    async def get_content_block_with_shortcut_categories(self) -> list[Any]:
        return await self.get_items_with_related('shortcut_categories')

    async def load_content_block_by_category_id(self, category_id: Any) -> list[Any]:
        return await self.load_items(category_id=category_id)

    async def filter_content_block_by_category_id(self, category_id: Any) -> list[Any]:
        return await self.filter_items(category_id=category_id)

    async def load_content_block_by_ids(self, ids: list[Any]) -> list[Any]:
        return await self.load_items_by_ids(ids)

    def add_computed_field(self, field: str) -> None:
        super().add_computed_field(field)

    def add_relation_field(self, field: str) -> None:
        super().add_relation_field(field)

    @property
    def active_content_blocks_ids(self) -> set[Any]:
        return self.active_item_ids



class ContentBlocksManager(ContentBlocksBase):
    _instance: ContentBlocksManager | None = None

    def __new__(cls, *args: Any, **kwargs: Any) -> ContentBlocksManager:
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self) -> None:
        super().__init__()

    async def _initialize_runtime_data(self, item: ContentBlocks) -> None:
        pass

content_blocks_manager_instance = ContentBlocksManager()